# -*- coding: utf-8 -*-


from collections import defaultdict
from operator import attrgetter
import pygame
import time

from .thing import Vec
from .picture import Obrazok
from .canvas import Platno
from .window import okno
from .events import _EventEmitterMixim
from .collisions import _CollisionEmitterMixim
from .exceptions import NemoznoVnoritSvet
from .exceptions import CyklickeVnorenieSveta


class Svet(Vec, _EventEmitterMixim, _CollisionEmitterMixim):

    def __init__(self, **kwargs):
        self._start_time = time.time()
        self._time = 0
        self._ticks = 0

        self._things_by_class = defaultdict(set)
        self._things = set()

        self._things_by_z = None
        self._recalculate_z = True

        _EventEmitterMixim.__init__(self)
        Vec.__init__(self, **kwargs)

        self.x_zarovnanie = 0
        self.y_zarovnanie = 0

    def _init_graphics(self):
        self._width = okno.sirka
        self._height = okno.vyska
        self.pozadie = Obrazok(self.sirka, self.vyska)

    @property
    def sirka(self):
        if self._world == self:
            # @HACK: Top worlds are kids of themselves
            return okno.sirka
        return self._width

    @sirka.setter
    def sirka(self, hodnota):
        if self._world == self:
            # @HACK: Top worlds are kids of themselves
            okno.sirka = hodnota
        else:
            self._width = hodnota

    @property
    def vyska(self):
        if self._world == self:
            # @HACK: Top worlds are kids of themselves
            return okno.vyska
        return self._height

    @vyska.setter
    def vyska(self, hodnota):
        if self._world == self:
            # @HACK: Top worlds are kids of themselves
            okno.vyska = hodnota
        else:
            self._height = hodnota

    @property
    def pozadie(self):
        return self._picture

    @pozadie.setter
    def pozadie(self, obrazok):
        self._picture = obrazok
        self._canvas = Platno(self._picture)

    @property
    def platno(self):
        return self._canvas

    @property
    def kroky(self):
        return self._ticks

    @property
    def cas(self):
        return self._time

    @property
    def veci(self):
        return list(self._things)

    @property
    def podsvety(self):
        return list(self._things_by_class[Svet])

    def _repaint(self, canvas):
        my_picture = Obrazok(self.sirka, self.vyska)
        my_canvas = Platno(my_picture)

        my_canvas.obrazok(
            self._picture,
            (0, 0),
        )

        if self._recalculate_z:
            self._things_by_z = list(
                sorted(self._things, key=lambda x: (x._z, x.id))
            )
            self._recalculate_z = False
        for thing in self._things_by_z:
            thing._repaint(my_canvas)

        canvas.obrazok(
            my_picture,
            (self.okraj_vlavo, self.okraj_hore),
        )

    def _connect_thing(self, thing):
        if thing == self:
            # @HACK: Top worlds are kids of themselves
            raise CyklickeVnorenieSveta()
        self._recalculate_z = True
        self._things.add(thing)
        for cls in thing.__class__.__mro__:
            self._things_by_class[cls].add(thing)

    def _disconnect_thing(self, thing):
        if thing == self:
            # @HACK: Top worlds are kids of themselves
            raise NemoznoVnoritSvet()
        self._recalculate_z = True
        self._things.remove(thing)
        for cls in thing.__class__.__mro__:
            self._things_by_class[cls].remove(thing)

    def _clear_things(self, thing):
        self._recalculate_z = True
        for thing in self._things:
            thing._world = None
        self._things.clear()
        for cls in self._things_by_class:
            self._things_by_class[cls].clear()

    def _activate(self):
        self._start_time = time.time() - self._time

    def _deactivate(self):
        self._time = time.time() - self._start_time

    def _tick(self):
        self._ticks += 1
        self._time = time.time() - self._start_time

        self._tick_collisions()
        self._tick_events()

        for subworld in self._things_by_class[Svet]:
            subworld._tick()



#  -*- coding: utf-8 -*-


from .thing_type import _ThingType


def zrazka(cls):
    def decorator(f):
        if not hasattr(f, "_gaminator_collisions"):
            f._gaminator_collisions = []
        f._gaminator_collisions.append(cls)
        return f
    return decorator


class _CollisionEmitterMixim(object):

    def _tick_collisions(self):
        calls = []
        for cls in self._things_by_class:
            if isinstance(cls, _ThingType):
                for clsi in cls._gaminator_collisions:
                    cls2 = _ThingType._get_class(clsi)
                    for fname in cls._gaminator_collisions[clsi]:
                        for thing in self._things_by_class[cls]:
                            if cls2 in self._things_by_class:
                                for thing2 in self._things_by_class[cls2]:
                                    if thing.prekryva_sa(thing2):
                                        calls.append(
                                            (getattr(thing, fname), thing2)
                                        )
        for f, thing2 in calls:
            f(thing2)



# -*- coding: utf-8 -*-


from gaminator import Vec, CIERNA, hra


class Obdl(Vec):

    def setup(self):
        self.farba = CIERNA
        self.vyfarbenie = False
        self.svet = hra.svet

    def paint(self, c):
        c.farba = self.farba
        c.rectangle(
            (0, 0),
            self.sirka, self.vyska,
            self.vyfarbenie
        )

    def step(self):
        self.repaint()


class Oval(Vec):

    def setup(self):
        self.farba = CIERNA
        self.vyfarbenie = False
        self.svet = hra.svet

    def paint(self, c):
        c.farba = self.farba
        c.ellipse(
            (self.sirka/2, self.vyska/2),
            self.sirka/2, self.vyska/2,
            self.vyfarbenie
        )

    def step(self):
        self.repaint()



#  -*- coding: utf-8 -*-


import pygame
from threading import Condition
import traceback

from .interactive import interact
from .picture import Picture
from .canvas import Canvas
from .window import window


class _Game:

    def __init__(self):
        self._end = False
        self._interactive = False
        self._lock = Condition()
        self._screen = None
        self._worlds = []
        self._world_changes = []
        self.tps = 60
        self.autoescape = True
        pass

    @property
    def world(self):
        return self._worlds[-1]

    def end(self):
        self._end = True

    def push_world(self, world):
        self._world_changes.append((1, world))

    def swap_world(self, world):
        self._world_changes.append((0, world))

    def pop_world(self):
        self._world_changes.append((-1, None))

    def start(self, world, interactive=False):
        self._interactive = interactive
        self.push_world(world)
        self._loop()

    def pressed(self, key):
        return self._pressed_keys[key]

    def _loop(self):

        if self._interactive:
            console = interact(self)

        pygame.init()

        window._apply_changes()
        self._screen = Picture(pygame.display.get_surface(), None)

        clock = pygame.time.Clock()

        try:
            with self._lock:
                while not self._end:

                    self._handle_world_changes()
                    if not self._worlds:
                        break

                    window._apply_changes()

                    if self._interactive:
                        self._lock.wait(0)

                    for event in pygame.event.get():
                        if event.type == pygame.QUIT:
                            self.end()
                        elif event.type == pygame.VIDEORESIZE:
                            window._height = event.h
                            window._width = event.w
                        elif event.type == pygame.KEYDOWN:
                            if (
                                self.autoescape and
                                event.key == pygame.K_ESCAPE
                            ):
                                self.end()

                            self._worlds[-1].event(
                                ("KEYDOWN", event.key), event.unicode
                            )
                            self._worlds[-1].event(
                                "KEYDOWN", event.key, event.unicode
                            )
                        elif event.type == pygame.KEYUP:
                            self._worlds[-1].event(
                                ("KEYUP", event.key)
                            )
                            self._worlds[-1].event(
                                "KEYUP", event.key
                            )

                    self._pressed_keys = pygame.key.get_pressed()

                    self._worlds[-1]._tick()

                    self._screen._surface.fill((255, 255, 255, 0))
                    self._worlds[-1]._repaint(Canvas(self._screen))

                    pygame.display.flip()
                    clock.tick(self.tps)
        except:
            traceback.print_exc()
        finally:
            pygame.quit()

    def _handle_world_changes(self):
        for action, world in self._world_changes:
            if self._worlds:
                self._worlds[-1]._deactivate()
            if action in [-1, 0] and self._worlds:
                w = self._worlds.pop()
                w._world = None  # @HACK: Topworlds are kids of themselves
            if action in [0, 1]:
                self._worlds.append(world)
                world.world = None  # Disconect world from its parent
                world._world = world  # @HACK: Topworlds are kids of themselves
            if self._worlds:
                self._worlds[-1]._activate()
        self._world_changes = []


game = _Game()



#  -*- coding: utf-8 -*-


import pygame


class _WindowSettings(object):

    def __init__(self):
        self._flags = 0
        self._width = 600
        self._height = 400
        self._changed_mode = True
        self._caption = "Gaminator"
        self._changed_caption = True
        self.fps = 60

    def _set_flag(self, flag):
        self._flags = self._flags | flag

    def _reset_flag(self, flag):
        self._flags = (self._flags | flag) - flag

    def _apply_changes(self):
        if self._changed_mode:
            self._changed_mode = False
            pygame.display.set_mode((self._width, self._height), self._flags)
        if self._changed_caption:
            self._changed_caption = False
            pygame.display.set_caption(self._caption)

    @property
    def sirka(self):
        return self._width

    @sirka.setter
    def sirka(self, _width):
        self._changed_mode = True
        self._width = _width

    @property
    def vyska(self):
        return self._height

    @vyska.setter
    def vyska(self, _height):
        self._changed_mode = True
        self._height = _height

    @property
    def nadpis(self):
        return self._caption

    @nadpis.setter
    def nadpis(self, _nazov):
        self._changed_caption = True
        self._caption = _nazov

    def cela_obrazovka(self):
        self._changed_mode = True
        self._flags = (
            pygame.FULLSCREEN | pygame.DOUBLEBUF |
            pygame.HWSURFACE | pygame.NOFRAME
        )

    def pevne(self):
        self._changed_mode = True
        self._flags = 0

    def nastavitelne(self):
        self._changed_mode = True
        self._flags = pygame.RESIZABLE


okno = _WindowSettings()



# -*- coding: utf-8 -*-


from .thing import Thing
from .canvas import Canvas
from .picture import Picture
from .exceptions import AssignmentUnsupported


class PictureThing(Thing):

    def _init_graphics(self):
        self.picture = Picture(20, 20)

    @property
    def width(self):
        return self._width

    @width.setter
    def width(self, value):
        raise AssignmentUnsupported()

    @property
    def height(self):
        return self._height

    @height.setter
    def height(self, value):
        raise AssignmentUnsupported()

    @property
    def picture(self):
        return self._picture

    @picture.setter
    def picture(self, picture):
        self._picture = picture
        self._canvas = Canvas(self._picture)
        self._width = picture.width
        self._height = picture.height

    @property
    def canvas(self):
        return self._canvas

    def _repaint(self, canvas):
        canvas.picture(
            self._picture,
            (self.border_left, self.border_up),
        )

    def paint(self, canvas):
        pass



# -*- coding: utf-8 -*-


from gaminator import Thing, BLACK, game


class Rect(Thing):

    def setup(self):
        self.color = BLACK
        self.filled = False
        self.world = game.world

    def paint(self, c):
        c.color = self.color
        c.rectangle(
            (0, 0),
            self.width, self.height,
            self.filled
        )

    def step(self):
        self.repaint()


class Oval(Thing):

    def setup(self):
        self.color = BLACK
        self.filled = False
        self.world = game.world

    def paint(self, c):
        c.color = self.color
        c.ellipse(
            (self.width/2, self.height/2),
            self.width/2, self.height/2,
            self.filled
        )

    def step(self):
        self.repaint()



#  -*- coding: utf-8 -*-


from .constants import CONSTANTS
import pygame
import pygame.gfxdraw


def norm(x):
    return int(round(x))


class Canvas:

    def __init__(self, picture, color=None):
        self.color = color or CONSTANTS['BLACK']
        self._surface = picture._surface

    def line(self, p1, p2):
        (x1, y1) = p1
        (x2, y2) = p2
        pygame.gfxdraw.line(
            self._surface, norm(x1), norm(y1), norm(x2), norm(y2),
            self.color,
        )

    def pixel(self, p):
        (x, y) = p
        pygame.gfxdraw.pixel(
            self._surface, norm(x), norm(y), self.color,
        )

    def rectangle(
        self, p, width, height, filled=False
    ):
        (x, y) = p
        params = (
            self._surface,
            (norm(x), norm(y), norm(width), norm(height)),
            self.color,
        )
        if filled:
            pygame.gfxdraw.box(*params)
        else:
            pygame.gfxdraw.rectangle(*params)

    def ellipse(self, p, rx, ry, filled=False):
        (x, y) = p
        params = (
            self._surface,
            norm(x), norm(y), norm(rx), norm(ry),
            self.color,
        )
        if filled:
            pygame.gfxdraw.filled_ellipse(*params)
        else:
            pygame.gfxdraw.ellipse(*params)

    def polygon(self, points, filled=False):
        params = (
            self._surface,
            list(map(lambda p: (norm(p[0]), norm(p[1])), points)),
            self.color,
        )
        if filled:
            pygame.gfxdraw.filled_polygon(*params)
        else:
            pygame.gfxdraw.polygon(*params)

    def picture(self, picture, p):
        (x, y) = p
        self._surface.blit(picture._surface, (norm(x), norm(y)))



#  -*- coding: utf-8 -*-


from .constants import CONSTANTS
import pygame
import pygame.gfxdraw


def norm(x):
    return int(round(x))


class Platno:

    def __init__(self, obrazok, farba=None):
        self.farba = farba or CONSTANTS['CIERNA']
        self._surface = obrazok._surface

    def ciara(self, p1, p2):
        (x1, y1) = p1
        (x2, y2) = p2
        pygame.gfxdraw.line(
            self._surface, norm(x1), norm(y1), norm(x2), norm(y2),
            self.farba,
        )

    def pixel(self, p):
        (x, y) = p
        pygame.gfxdraw.pixel(
            self._surface, norm(x), norm(y), self.farba,
        )

    def obdlzdnik(
        self, p, sirka, vyska, vyfarbenie=False
    ):
        (x, y) = p
        params = (
            self._surface,
            (norm(x), norm(y), norm(sirka), norm(vyska)),
            self.farba,
        )
        if vyfarbenie:
            pygame.gfxdraw.box(*params)
        else:
            pygame.gfxdraw.rectangle(*params)

    def elipsa(self, p, rx, ry, vyfarbenie=False):
        (x, y) = p
        params = (
            self._surface,
            norm(x), norm(y), norm(rx), norm(ry),
            self.farba,
        )
        if vyfarbenie:
            pygame.gfxdraw.filled_ellipse(*params)
        else:
            pygame.gfxdraw.ellipse(*params)

    def mnohouholnik(self, body, vyfarbenie=False):
        params = (
            self._surface,
            list(map(lambda p: (norm(p[0]), norm(p[1])), body)),
            self.farba,
        )
        if vyfarbenie:
            pygame.gfxdraw.filled_polygon(*params)
        else:
            pygame.gfxdraw.polygon(*params)

    def obrazok(self, obrazok, p):
        (x, y) = p
        self._surface.blit(obrazok._surface, (norm(x), norm(y)))



# -*- coding: utf-8 -*-

from .constants import CONSTANTS

from .game import game
from .thing import Thing
from .world import World

from .events import event
from .collisions import collision

from .color import Color
from .picture import Picture, open_picture, text_to_picture
from .canvas import Canvas
from .picture_thing import PictureThing

from .window import window


import pygame as _pygame
for name in dir(_pygame):
    if name.startswith("K_"):
        CONSTANTS[name] = getattr(_pygame, name)

import sys as _sys
_module = _sys.modules[__name__]
for key, value in CONSTANTS.items():
    setattr(_module, key, value)

_pygame.font.init()



#  -*- coding: utf-8 -*-


from code import InteractiveConsole
from threading import Thread
import traceback
import readline
import rlcompleter
import sys

from .thing_type import _ThingType


class _GaminatorInteractiveConsole(InteractiveConsole):

    def __init__(self, game):
        self.game = game
        locals = {}
        exec('from gaminator import *', locals)
        locals.update(_ThingType._classes)
        readline.set_completer(rlcompleter.Completer(locals).complete)
        readline.parse_and_bind("tab: complete")
        InteractiveConsole.__init__(self, locals)

    def runcode(self, code):
        with self.game._lock:
            InteractiveConsole.runcode(self, code)


def interact(game):
    console = _GaminatorInteractiveConsole(game)

    def run():
        try:
            console.interact()
        finally:
            with game._lock:
                game.koniec()
    thread = Thread(target=run)
    thread.start()

    return console



#  -*- coding: utf-8 -*-


import heapq

from .thing_type import _ThingType


def udalost(name):
    def decorator(f):
        if not hasattr(f, "_gaminator_events"):
            f._gaminator_events = []
        f._gaminator_events.append(name)
        return f
    return decorator


class _EventEmitterMixim(object):

    def __init__(self):
        self._events_queue = []
        self._events_queue_id = 0

    def nacasuj_udalost(self, cas, udalost, *args, **kwargs):
        heapq.heappush(self._events_queue, (
            self.cas + cas, self._events_queue_id,
            udalost, args, kwargs,
        ))

        self._events_queue_id += 1

        for subworld in self.podsvety:
            subworld.nacasuj_udalost(
                cas, udalost, *args, **kwargs
            )

    def udalost(self, udalost, *args, **kwargs):
        self.nacasuj_udalost(0, udalost, *args, **kwargs)

    def _tick_events(self):

        heapq.heappush(self._events_queue, (
            self.cas, -1, 'KROK', [], {},
        ))

        calls = []

        while self._events_queue and self._events_queue[0][0] <= self.cas:
            (_time, _id, event, args, kwargs) = self._events_queue[0]
            for cls in self._things_by_class:
                if isinstance(cls, _ThingType):
                    for fname in cls._gaminator_events[event]:
                        for thing in self._things_by_class[cls]:
                            calls.append((getattr(thing, fname), args, kwargs))
            for cls in self.__class__.mro():
                if isinstance(cls, _ThingType):
                    for fname in cls._gaminator_events[event]:
                        calls.append((getattr(self, fname), args, kwargs))
            heapq.heappop(self._events_queue)

        for f, args, kwargs in calls:
            f(*args, **kwargs)



# -*- coding: utf-8 -*-


from .thing_type import _ThingType
from .picture import Obrazok
from .canvas import Platno


# Python 2 and 3 compatible way to define metaclass
def _with_metaclass(mcls):
    def decorator(cls):
        body = vars(cls).copy()
        body.pop('__dict__', None)
        body.pop('__weakref__', None)
        return mcls(cls.__name__, cls.__bases__, body)
    return decorator


@_with_metaclass(_ThingType)
class Vec(object):

    _next_id = 1

    def __init__(self, **kwargs):
        self._world = None

        self.id = Vec._next_id
        Vec._next_id += 1

        self.x = 0
        self.y = 0
        self._z = 0
        self.x_zarovnanie = 0.5
        self.y_zarovnanie = 0.5

        self._init_graphics()

        for key in kwargs:
            setattr(self, key, kwargs[key])

        calls = []

        for cls in self.__class__.mro():
            if isinstance(cls, _ThingType):
                for fname in cls._gaminator_events['NASTAV']:
                    calls.append(getattr(self, fname))

        for f in calls:
            f()

    def _init_graphics(self):
        self._width = 20
        self._height = 20
        self._will_repaint = True
        self._will_resize = True
        self._picture = Obrazok(20, 20)

    @property
    def svet(self):
        return self._world

    @svet.setter
    def svet(self, value):
        if self._world is not None:
            self._world._disconnect_thing(self)
        if value is not None:
            value._connect_thing(self)
        self._world = value

    @svet.deleter
    def svet(self):
        if self._world is not None:
            self._world._disconnect_thing(self)
        self._world = None

    @property
    def z(self):
        return self._z

    @z.setter
    def z(self, z):
        self._z = z
        if self._world is not None:
            self._world._recalculate_z = True

    @property
    def sirka(self):
        return self._width

    @sirka.setter
    def sirka(self, sirka):
        self._will_resize = True
        self._width = sirka

    @property
    def vyska(self):
        return self._height

    @vyska.setter
    def vyska(self, vyska):
        self._will_resize = True
        self._height = vyska

    @property
    def okraj_vlavo(self):
        return self.x + (0-self.x_zarovnanie)*self._width

    @property
    def okraj_vpravo(self):
        return self.x + (1-self.x_zarovnanie)*self._width

    @property
    def okraj_hore(self):
        return self.y + (0-self.y_zarovnanie)*self._height

    @property
    def okraj_dole(self):
        return self.y + (1-self.y_zarovnanie)*self._height

    def prekryva_sa(self, druhy):
        return (
            max(
                self.okraj_vlavo,
                druhy.okraj_vlavo,
            ) < min(
                self.okraj_vpravo,
                druhy.okraj_vpravo,
            ) and max(
                self.okraj_hore,
                druhy.okraj_hore,
            ) < min(
                self.okraj_dole,
                druhy.okraj_dole,
            )
        )

    def prekresli(self):
        self._will_repaint = True

    def _repaint(self, canvas):
        if self._will_resize:
            self._will_repaint = True
            self._picture.zresetuj(
                sirka=self._width, vyska=self._height,
            )
            self._will_resize = False
        if self._will_repaint:
            self._picture.zmaz()
            self.vykresli(Platno(self._picture))
            self._will_repaint = False
        canvas.obrazok(
            self._picture,
            (self.okraj_vlavo, self.okraj_hore),
        )

    def setup(self, *args, **kwargs):
        pass

    def step(self):
        pass

    def vykresli(self, platno):
        pass



# -*- coding: utf-8 -*-


from collections import defaultdict
from operator import attrgetter
import pygame
import time

from .thing import Thing
from .picture import Picture
from .canvas import Canvas
from .window import window
from .events import _EventEmitterMixim
from .collisions import _CollisionEmitterMixim
from .exceptions import TopWorldSubworlding
from .exceptions import SelfSubworlding


class World(Thing, _EventEmitterMixim, _CollisionEmitterMixim):

    def __init__(self, **kwargs):
        self._start_time = time.time()
        self._time = 0
        self._ticks = 0

        self._things_by_class = defaultdict(set)
        self._things = set()

        self._things_by_z = None
        self._recalculate_z = True

        _EventEmitterMixim.__init__(self)
        Thing.__init__(self, **kwargs)

        self.x_align = 0
        self.y_align = 0

    def _init_graphics(self):
        self._width = window.width
        self._height = window.height
        self.background = Picture(self.width, self.height)

    @property
    def width(self):
        if self._world == self:
            # @HACK: Top worlds are kids of themselves
            return window.width
        return self._width

    @width.setter
    def width(self, value):
        if self._world == self:
            # @HACK: Top worlds are kids of themselves
            window.width = value
        else:
            self._width = value

    @property
    def height(self):
        if self._world == self:
            # @HACK: Top worlds are kids of themselves
            return window.height
        return self._height

    @height.setter
    def height(self, value):
        if self._world == self:
            # @HACK: Top worlds are kids of themselves
            window.height = value
        else:
            self._height = value

    @property
    def background(self):
        return self._picture

    @background.setter
    def background(self, picture):
        self._picture = picture
        self._canvas = Canvas(self._picture)

    @property
    def canvas(self):
        return self._canvas

    @property
    def ticks(self):
        return self._ticks

    @property
    def time(self):
        return self._time

    @property
    def things(self):
        return list(self._things)

    @property
    def subworlds(self):
        return list(self._things_by_class[World])

    def _repaint(self, canvas):
        my_picture = Picture(self.width, self.height)
        my_canvas = Canvas(my_picture)

        my_canvas.picture(
            self._picture,
            (0, 0),
        )

        if self._recalculate_z:
            self._things_by_z = list(
                sorted(self._things, key=lambda x: (x._z, x.id))
            )
            self._recalculate_z = False
        for thing in self._things_by_z:
            thing._repaint(my_canvas)

        canvas.picture(
            my_picture,
            (self.border_left, self.border_up),
        )

    def _connect_thing(self, thing):
        if thing == self:
            # @HACK: Top worlds are kids of themselves
            raise SelfSubworlding()
        self._recalculate_z = True
        self._things.add(thing)
        for cls in thing.__class__.__mro__:
            self._things_by_class[cls].add(thing)

    def _disconnect_thing(self, thing):
        if thing == self:
            # @HACK: Top worlds are kids of themselves
            raise TopWorldSubworlding()
        self._recalculate_z = True
        self._things.remove(thing)
        for cls in thing.__class__.__mro__:
            self._things_by_class[cls].remove(thing)

    def _clear_things(self, thing):
        self._recalculate_z = True
        for thing in self._things:
            thing._world = None
        self._things.clear()
        for cls in self._things_by_class:
            self._things_by_class[cls].clear()

    def _activate(self):
        self._start_time = time.time() - self._time

    def _deactivate(self):
        self._time = time.time() - self._start_time

    def _tick(self):
        self._ticks += 1
        self._time = time.time() - self._start_time

        self._tick_collisions()
        self._tick_events()

        for subworld in self._things_by_class[World]:
            subworld._tick()



#  -*- coding: utf-8 -*-


from . import color
from .constants import CONSTANTS
import pygame
import os


class Obrazok:

    def __init__(self, sirka, vyska, je_priesvitny=False):

        self._width = 0
        self._height = 0
        self._has_alpha = False

        if isinstance(sirka, pygame.Surface) and vyska is None:
            self._set_surface(sirka)
        else:
            self.zresetuj(sirka, vyska, je_priesvitny)

    def zresetuj(
        self, sirka=None, vyska=None, je_priesvitny=None
    ):
        if sirka is not None:
            self._width = sirka
        if vyska is not None:
            self._height = vyska
        if je_priesvitny is not None:
            self._has_alpha = je_priesvitny

        dims = (self._width, self._height)

        flags = 0
        if self._has_alpha:
            self._surface = pygame.Surface(dims, pygame.SRCALPHA, 32)
            self._surface.convert_alpha()
        else:
            self._surface = pygame.Surface(dims)
            self._surface.set_colorkey((0, 1, 0))

        self.zmaz()

    def zmaz(self):
        self._surface.fill((0, 1, 0, 0))

    @property
    def sirka(self):
        return self._width

    @property
    def vyska(self):
        return self._height

    @property
    def je_priesvitny(self):
        return self._has_alpha

    def _set_surface(self, surface):
        self._surface = surface
        self._width = surface.get_width()
        self._height = surface.get_height()


def obrazok_zo_suboru(*path):
    surface = pygame.image.load(os.path.join(*path))
    return Obrazok(surface, None)


def obrazok_z_textu(
    text="", velkost=10, farba=CONSTANTS["CIERNA"],
    hrube=False, sikme=False,
):
    font = pygame.font.SysFont(
        None, velkost, bold=hrube, italic=sikme,
    )
    surface = font.render(text, False, farba)
    return Obrazok(surface, None)



#  -*- coding: utf-8 -*-


from . import color
from .constants import CONSTANTS
import pygame
import os


class Picture:

    def __init__(self, width, height, has_alpha=False):

        self._width = 0
        self._height = 0
        self._has_alpha = False

        if isinstance(width, pygame.Surface) and height is None:
            self._set_surface(width)
        else:
            self.reset(width, height, has_alpha)

    def reset(
        self, width=None, height=None, has_alpha=None
    ):
        if width is not None:
            self._width = width
        if height is not None:
            self._height = height
        if has_alpha is not None:
            self._has_alpha = has_alpha

        dims = (self._width, self._height)

        flags = 0
        if self._has_alpha:
            self._surface = pygame.Surface(dims, pygame.SRCALPHA, 32)
            self._surface.convert_alpha()
        else:
            self._surface = pygame.Surface(dims)
            self._surface.set_colorkey((0, 1, 0))

        self.clear()

    def clear(self):
        self._surface.fill((0, 1, 0, 0))

    @property
    def width(self):
        return self._width

    @property
    def height(self):
        return self._height

    @property
    def has_alpha(self):
        return self._has_alpha

    def _set_surface(self, surface):
        self._surface = surface
        self._width = surface.get_width()
        self._height = surface.get_height()


def open_picture(*path):
    surface = pygame.image.load(os.path.join(*path))
    return Picture(surface, None)


def text_to_picture(
    text="", size=10, color=CONSTANTS["BLACK"],
    bold=False, italic=False,
):
    font = pygame.font.SysFont(
        None, size, bold=bold, italic=italic,
    )
    surface = font.render(text, False, color)
    return Picture(surface, None)



# -*- coding: utf-8 -*-

import os
from setuptools import setup
from setuptools.command.sdist import sdist

LOCALES = [
    ('default.py', 'gaminator'),
    ('sk.py', 'gaminator_sk'),
]

PACKAGES = [
    '%s',
    '%s.starter',
]

DESCRIPTION = """A library for easy and fast development of simple games.
For educational purposes.
"""


def get_packages():
    for _, dest_root in LOCALES:
        for package in PACKAGES:
            yield package % dest_root


class TranslateSDistCommand(sdist):

    def run(self):
        from localization.tool import localize

        ROOT = os.path.dirname(__file__)
        for trans_file, dest_root in LOCALES:
            trans_file = os.path.join(ROOT, 'localization', 'data', trans_file)
            dest_root = os.path.join(ROOT, dest_root)
            src_root = os.path.join(ROOT, 'gaminator-src')
            localize(src_root, dest_root, trans_file)
        sdist.run(self)


setup(
    name="gaminator",
    version="0.1.3",
    author=u"Marián Horňák",
    author_email="marian.sysel.hornak@gmail.com",
    description=DESCRIPTION,
    license="MIT",
    keywords=["education", "game"],
    url="https://github.com/syslo/gaminator",
    download_url="https://github.com/syslo/gaminator/tarball/v0.1.3",
    packages=list(get_packages()),
    classifiers=[
        "Development Status :: 4 - Beta",
    ],
    cmdclass={
        'sdist': TranslateSDistCommand,
    },
)



# -*- coding: utf-8 -*-


from .thing import Vec
from .canvas import Platno
from .picture import Obrazok
from .exceptions import NepodporovanePriradenie


class ObrazkovaVec(Vec):

    def _init_graphics(self):
        self.obrazok = Obrazok(20, 20)

    @property
    def sirka(self):
        return self._width

    @sirka.setter
    def sirka(self, hodnota):
        raise NepodporovanePriradenie()

    @property
    def vyska(self):
        return self._height

    @vyska.setter
    def vyska(self, hodnota):
        raise NepodporovanePriradenie()

    @property
    def obrazok(self):
        return self._picture

    @obrazok.setter
    def obrazok(self, obrazok):
        self._picture = obrazok
        self._canvas = Platno(self._picture)
        self._width = obrazok.sirka
        self._height = obrazok.vyska

    @property
    def platno(self):
        return self._canvas

    def _repaint(self, canvas):
        canvas.obrazok(
            self._picture,
            (self.okraj_vlavo, self.okraj_hore),
        )

    def vykresli(self, platno):
        pass



# -*- coding: utf-8 -*-


from collections import defaultdict


class _ThingType(type):

    _classes = dict()

    def __init__(cls, name, bases, dct):
        super(_ThingType, cls).__init__(name, bases, dct)
        cls._gaminator_events = defaultdict(list)
        cls._gaminator_collisions = defaultdict(list)
        _ThingType._classes[name] = cls
        for k in dct:
            cls._new_attribute(k, dct[k])

    def _new_attribute(cls, key, value):
        if hasattr(value, '__call__') and hasattr(value, '_gaminator_events'):
            for event in value._gaminator_events:
                cls._gaminator_events[event].append(key)
            del value._gaminator_events
        if hasattr(value, '__call__') and hasattr(value, '_gaminator_collisions'):
            for clsi in value._gaminator_collisions:
                cls._gaminator_collisions[clsi].append(key)
            del value._gaminator_collisions

    @ staticmethod
    def _get_class(clsi):
        if isinstance(clsi, _ThingType):
            return clsi
        return _ThingType._classes[clsi]



# -*- coding: utf-8 -*-


from .thing_type import _ThingType
from .picture import Picture
from .canvas import Canvas


# Python 2 and 3 compatible way to define metaclass
def _with_metaclass(mcls):
    def decorator(cls):
        body = vars(cls).copy()
        body.pop('__dict__', None)
        body.pop('__weakref__', None)
        return mcls(cls.__name__, cls.__bases__, body)
    return decorator


@_with_metaclass(_ThingType)
class Thing(object):

    _next_id = 1

    def __init__(self, **kwargs):
        self._world = None

        self.id = Thing._next_id
        Thing._next_id += 1

        self.x = 0
        self.y = 0
        self._z = 0
        self.x_align = 0.5
        self.y_align = 0.5

        self._init_graphics()

        for key in kwargs:
            setattr(self, key, kwargs[key])

        calls = []

        for cls in self.__class__.mro():
            if isinstance(cls, _ThingType):
                for fname in cls._gaminator_events['SETUP']:
                    calls.append(getattr(self, fname))

        for f in calls:
            f()

    def _init_graphics(self):
        self._width = 20
        self._height = 20
        self._will_repaint = True
        self._will_resize = True
        self._picture = Picture(20, 20)

    @property
    def world(self):
        return self._world

    @world.setter
    def world(self, value):
        if self._world is not None:
            self._world._disconnect_thing(self)
        if value is not None:
            value._connect_thing(self)
        self._world = value

    @world.deleter
    def world(self):
        if self._world is not None:
            self._world._disconnect_thing(self)
        self._world = None

    @property
    def z(self):
        return self._z

    @z.setter
    def z(self, z):
        self._z = z
        if self._world is not None:
            self._world._recalculate_z = True

    @property
    def width(self):
        return self._width

    @width.setter
    def width(self, width):
        self._will_resize = True
        self._width = width

    @property
    def height(self):
        return self._height

    @height.setter
    def height(self, height):
        self._will_resize = True
        self._height = height

    @property
    def border_left(self):
        return self.x + (0-self.x_align)*self._width

    @property
    def border_right(self):
        return self.x + (1-self.x_align)*self._width

    @property
    def border_up(self):
        return self.y + (0-self.y_align)*self._height

    @property
    def border_down(self):
        return self.y + (1-self.y_align)*self._height

    def collides(self, other):
        return (
            max(
                self.border_left,
                other.border_left,
            ) < min(
                self.border_right,
                other.border_right,
            ) and max(
                self.border_up,
                other.border_up,
            ) < min(
                self.border_down,
                other.border_down,
            )
        )

    def repaint(self):
        self._will_repaint = True

    def _repaint(self, canvas):
        if self._will_resize:
            self._will_repaint = True
            self._picture.reset(
                width=self._width, height=self._height,
            )
            self._will_resize = False
        if self._will_repaint:
            self._picture.clear()
            self.paint(Canvas(self._picture))
            self._will_repaint = False
        canvas.picture(
            self._picture,
            (self.border_left, self.border_up),
        )

    def setup(self, *args, **kwargs):
        pass

    def step(self):
        pass

    def paint(self, canvas):
        pass



# -*- coding: utf-8 -*-

from .constants import CONSTANTS

from .game import hra
from .thing import Vec
from .world import Svet

from .events import udalost
from .collisions import zrazka

from .color import Farba
from .picture import Obrazok, obrazok_zo_suboru, obrazok_z_textu
from .canvas import Platno
from .picture_thing import ObrazkovaVec

from .window import okno


import pygame as _pygame
for name in dir(_pygame):
    if name.startswith("K_"):
        CONSTANTS[name] = getattr(_pygame, name)

import sys as _sys
_module = _sys.modules[__name__]
for key, value in CONSTANTS.items():
    setattr(_module, key, value)

_pygame.font.init()



#  -*- coding: utf-8 -*-


from .constants import CONSTANTS


class Color(object):

    def __init__(self, r, g, b, a=255):
        self._r = int(r)
        self._g = int(g)
        self._b = int(b)
        self._a = int(a)

        # @HACK: Internal colorkey
        if r == 0 and g == 1 and b == 0:
            self._g = 0

    @property
    def r(self):
        return self._r

    @property
    def g(self):
        return self._g

    @property
    def b(self):
        return self._b

    @property
    def a(self):
        return self._a

    def _tuple(self):
        return (
            max(min(self._r, 255), 0),
            max(min(self._g, 255), 0),
            max(min(self._b, 255), 0),
            max(min(self._a, 255), 0)
        )

    def __len__(self):
        return 4

    def __getitem__(self, index):
        return self._tuple()[index]

    def __mul__(self, other):
        return Color(
            self._r * other, self._g * other, self._b * other, self._a * other
        )

    def __add__(self, other):
        return Color(
            self._r + other._r, self._g + other._g,
            self._b + other._b, self._a + other._a
        )

    def __rmul__(self, other):
        return self * other

    def modified(self, r=None, g=None, b=None, a=None):
        return Color(
            r if r is not None else self._r, g if g is not None else self._g,
            b if b is not None else self._b, a if a is not None else self._a,
        )

    def mixed(self, other, ratio=0.5):
        return self * ratio + other * (1 - ratio)


CONSTANTS['BLACK'] = Color(0, 0, 0)
CONSTANTS['WHITE'] = Color(255, 255, 255)
CONSTANTS['BLUE'] = Color(0, 0, 255)
CONSTANTS['GREEN'] = Color(0, 255, 0)
CONSTANTS['RED'] = Color(255, 0, 0)
CONSTANTS['YELLOW'] = Color(255, 255, 0)
